/* ===========================================================================
    CMF Programming functions for PB-555-ED board utilities

    Version:        1.0
    Author:         Dusty Nidey, Axiom Mfg.
	Compiler:		Diab Data Compiler version 4.2

============================================================================*/

#include "cmf_parallel_descriptor.h"
#include "interface.h"
#include "diab.h"
#include "555cmf.h"
#include "serial.h"

VUNS32 	blockmask;
UINT32 	*TABLE;

UINT32	PageOffset ;

#define OUTBUFSIZE	64
UINT8	outbuf[OUTBUFSIZE];

void (* RomFunction)();				// this is a function pointer that holds the starting address in rom to execute


#define	MACRO_DEFPOINTER(FUNC_NAME)	\
	InputStruct_ ## FUNC_NAME * inParamsBuf = (InputStruct_ ## FUNC_NAME *) TABLE[INDEX_pInputBuffer];	\
	OutputStruct_ ## FUNC_NAME * outp = (OutputStruct_ ## FUNC_NAME *) TABLE[INDEX_pOutputBuffer];		\
	inParamsBuf->size = sizeof(InputStruct_ ## FUNC_NAME) - sizeof(UINT32);


void TableInit(UINT32 driver_addr){
	TABLE = (UINT32*)driver_addr;   /* assign ptr to driver address */

    TABLE[INDEX_CallinGate]+=(UINT32)driver_addr; 	/* update the zero-based function offsets */
    TABLE[INDEX_pInputBuffer]+=(UINT32)driver_addr;
    TABLE[INDEX_pOutputBuffer]+=(UINT32)driver_addr;
}

UINT32 failmessage(char *msg, UINT32 code){
	puts("\n");
	puts(msg);
	puts(" Failed.  Status Code ");
	puth(code,8);
	puts("  ");
	switch(code){
		case CMF_ERROR_NO_VPP:
			puts("NO VPP - check jumper");
			break;
		case PTRD_ERROR_VERIFY:
			puts("Verify Failed");
			break;
	}
	return code;
}

void Callout(void){

	// push all
			asm("	stwu    r1, -0x78(r1)		");	// Backup register r2-r31 to Stack
			asm("	stmw    r2, 0(r1)			");
			asm("	stwu    r1, -0x4(r1)		"); // Backup register r0 to Stack
			asm("	stw		r0,0(r1)			");

	// call
	((UINT32(*)(void))(TABLE[INDEX_CallinGate])) ();

	// pop all
			asm("	lwz     r0, 0(r1)			"); // Restore register r0 from Stack
			asm("	addi    r1, r1, 0x04		");
			asm("	lmw     r2, 0(r1)			"); // Restore register r2-r31 from Stack
			asm("	addi    r1, r1, 0x78		");
}

tCMF_DESCRIPTOR* GetDescriptor(void){
	MACRO_DEFPOINTER(GetDescriptor)

	// fill out buffer
	inParamsBuf->size = sizeof(InputStruct_GetDescriptor) - sizeof(UINT32);
	inParamsBuf->command = COMMAND_GETDESCRIPTOR;

	Callout();	// call out

	// return
	if (outp->returnedValue==CMF_OK) {
		return outp->descriptor;
	}
	else
		return NULL;
}

UINT32 ParallelErase( UINT32 enabledBlocks ){

	MACRO_DEFPOINTER(ParallelErase)

	// fill out buffer
	inParamsBuf->size = sizeof(InputStruct_ParallelErase) - sizeof(UINT32);
	inParamsBuf->command = COMMAND_PARALLELERASE;
	inParamsBuf->enabledBlocks = enabledBlocks;

	Callout();	// call out

	return outp->returnedValue;
}

//  Erase all CMF memory
UINT32 CMF_EraseAll(){
	UINT32 	status;
	UINT32 	offset;
	UINT32 	vfail_addr, vfail_data, vfail_srcdata;
	UINT32 	modBaseAddress = 0;

	// Erase all blocks including shadow
	puts("\nErasing Flash...");
	status = ParallelErase (blockmask);
	if (status != CMF_OK){
		return (failmessage("Erase",status));
	}
	return CMF_OK;
}


UINT32 ParallelProgram( UINT32 *src, UINT32 enabledBlocks, UINT32 offset, UINT32 pagesetNum, UINT32 shadow ){

	MACRO_DEFPOINTER(ParallelProgram)

	// fill out buffer
	inParamsBuf->size = sizeof(InputStruct_ParallelProgram) - sizeof(UINT32);
	inParamsBuf->command = COMMAND_PARALLELPROGRAM;
	inParamsBuf->src = src;
	inParamsBuf->enabledBlocks = enabledBlocks;
	inParamsBuf->offset = offset;
	inParamsBuf->pagesetNum = pagesetNum;
	inParamsBuf->shadow = shadow;

	Callout();	// call out

	return outp->returnedValue;
}

// Blockmask and Page calculation
// This will take the offset address and set the blockmask for parallel programming so that only a single block is programmed.
// This will also take the offset address and set the page offset for the block to be programmed in parallel mode
// The blockmask is returned and the global PageOffset is set to this blocks offset
VUNS32 CMF_BlockCalc(UINT32 offset){
	UINT32 checkval;
	VUNS32 blockval;

	blockval = BLOCK_A0;
	checkval = 0x0;

	while(checkval < 0x70000){
		if(offset < (checkval + 0x8000)){
			PageOffset = offset - checkval;	// calculate this blocks page address
			return(blockval);
		}
		checkval += 0x8000;
		blockval >>= 1;
	}
	return(0);	// here if an invalid offset value
}

//----------------------------------------------------------------------------
UINT32 CMF_ProgramPage(UINT32 *bfr, UINT32 offset){
	UINT32 	status;
	UINT8	count1;
	UINT32 *ptr;

	if(offset < 0x10000){
		return(CMF_ERROR_PROTECTED);	// The memory you are trying to program is in protected space and cannot be programed with this software.  See the board documention for more information.
	}
	blockmask = CMF_BlockCalc(offset);
	if(blockmask > 0){
		// buffer = source data address
		// blockmask = mask of blocks that are enabled to be parallel programmed (buffer must have data at each page)
		// offset = offset in this block to be programmed
		// 1 = number of pages to be written to a block
		// 0 = normal address space (1=shadow space)

		status = ParallelProgram (bfr,blockmask,PageOffset,1,0);
		if(status != CMF_OK)	return(status);

		// verify we can read back what was programmed
		ptr = (UINT32 *) offset;
		for(count1=0; count1<0x10; count1++){
			if(bfr[count1] != ptr[count1])	return(PTRD_ERROR_VERIFY);
		}
	}

	return(CMF_OK);
}
//---------------------------------------------------------------------------
UINT32 CMF_ProgramFile(){
	int count, count1, bcount, ipos;
	int addrh, addrl;
	long address, cur_address, new_address;
	char firstflag;
	UINT32 	status;

	puts("\n\n Send Motorola S19 File Now...");
	firstflag = 0;
	echo = FALSE;

	// clear output buffer
	for(count1=0; count1 < OUTBUFSIZE; count1++)	outbuf[count1] = 0xff;

	count = 0; addrl = 0; addrh = 0; cur_address = 0; address = 0;


	for( ; ;){

		GetString();		// get a string from the serial port

		if((inbuf[0] != 'S') && (inbuf[0] != 's')){
			puts("\n\n ERROR: Invalid S-Record Format.\n\n");
			return(1);
		}
		if(inbuf[1] == '3'){
			bcount = StrHexNum(&inbuf[2], 1);	// get byte count
			bcount -=5;	// remove address and checksum from the count
			bcount *=2;	// adjust for ascii
			addrh = StrHexNum(&inbuf[4], 2);	// get high address word
			addrl = StrHexNum(&inbuf[8], 2);	// get low address word
			new_address = (addrh * 0x10000) + addrl;

			if(firstflag == 0){
				cur_address = new_address;
				address = new_address;
				firstflag = 1;
			}

			ipos = 0;	// start of input data
			do{
				if(count > 0 && new_address > cur_address){ // if there's a gap in the s record
					outbuf[count] = 0xFF;
				}
				else{
					outbuf[count] = StrHexNum(&inbuf[ipos+12], 1);
					ipos+=2;
				}
				++cur_address;
				++count;
				if(count > 63){	// if end of buffer reached, program it
					status = CMF_ProgramPage((UINT32 *) outbuf, address);	// program this page
					if (status != CMF_OK){
						failmessage("Programming",status);
						delay(100);	// delay in mil sec.
						ClearSerial();
						return(status);
					}

//                    if(new_address >= (cur_address + 0xFF)){    // if large gap, skip to next block in s-record
                    if(new_address >= (address + 0xFF)){    // if large gap, skip to next block in s-record
						cur_address = new_address;
					}
					address = cur_address;
					count=0;
					for(count1=0; count1 < OUTBUFSIZE; count1++)	outbuf[count1] = 0xff;	// clear output buffer
				}
			}while(ipos < bcount);
		}

		//		scrchar = outputscroll(scrchar);	// displays a rotating slash character
		putchar('.');

		if(inbuf[1] == '7' || inbuf[1] == '9'){	// end of s19 record
			if(count > 0){
				status = CMF_ProgramPage((UINT32 *) outbuf, address);	// program this page
				if (status != CMF_OK)
 					failmessage("Programming",status);
			}
			delay(50);	// delay in mil sec.
			ClearSerial();
			return(0);
		}
	}
}

//----------------------------------------------------------------------------
// CMF Programming menu for the 555-ED single-chip board

#define TRD_ARRAY_SIZE	0x21F2
//unsigned char TRD_RAM_ARRAY[TRD_ARRAY_SIZE];    // global array to hold CMF programming code
unsigned char *TRD_RAM_ARRAY;    // global array to hold CMF programming code

void CMF_Menu(){
	UNS8 choice;
	UNS16	count1;

    // for this version, we must move the programming array to internal RAM
    // because the CMFI memory device cannot program itself from the same memory device
	puts("\n\nInitializing and moving Parallel TRD Array...");

    TRD_RAM_ARRAY = (unsigned char *) 0x3FC000;           // point to the bottom of internal SRAM B array
    for(count1=0; count1 < TRD_ARRAY_SIZE; count1++){
		TRD_RAM_ARRAY[count1] = PARALLEL_TRD_ARRAY[count1];
	}
	TableInit( (UINT32) TRD_RAM_ARRAY );

	for( ; ;){
		puts("\n\n 555-ED single-chip board - CMF Flash Programming\n\n");
		puts("\n --------------------------------------------------");
		puts("\n These functions require the VPP_EN jumper to be installed.\n");
		puts("\n E. Erase CMF Flash");
		puts("\n P. Program CMF Flash from file");
		puts("\n T. Test run your program");
		puts("\n Q. Quit to Menu");

		puts("\n\n Select: ");
		choice = tolower(GetSByte());
		puts("\n\n");
		switch(choice){
			case 'e':
                blockmask = BLOCK_EDALL; // select all blocks that are allowed by this software
                CMF_EraseAll(); // Erase CMF memory
				puts("\n\n --- If finished remove VPP_EN to protect the board. ---\n");
				break;

			case 'p':
				CMF_ProgramFile();
				puts("\n\n --- If finished programming, remove VPP_EN to protect the board. ---\n");
				puts(" To boot your program, remove the PROG_EN jumper.\n");
				break;

			case 't':
				RomFunction = (void (*)()) 0x10100;		// address of user program's reset vector
				RomFunction();							// go execute their code
				break;

            case 'q':
				return;
		}
	}
}
